// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: userinput.proto
// Protobuf C++ Version: 7.36.1

#ifndef userinput_2eproto_2epb_2eh
#define userinput_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7036001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_userinput_2eproto

// Internal implementation detail -- do not use these members.
struct TableStruct_userinput_2eproto {
  static const ::uint32_t offsets[];
};
namespace ClientBuffers {
class Instruction;
struct InstructionGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern InstructionGlobalsTypeInternal Instruction_globals_;
extern const ::google::protobuf::internal::ClassDataLite Instruction_class_data_;
#else
extern const InstructionGlobalsTypeInternal Instruction_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Keystroke;
struct KeystrokeGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern KeystrokeGlobalsTypeInternal Keystroke_globals_;
extern const ::google::protobuf::internal::ClassDataLite Keystroke_class_data_;
#else
extern const KeystrokeGlobalsTypeInternal Keystroke_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ResizeMessage;
struct ResizeMessageGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ResizeMessageGlobalsTypeInternal ResizeMessage_globals_;
extern const ::google::protobuf::internal::ClassDataLite ResizeMessage_class_data_;
#else
extern const ResizeMessageGlobalsTypeInternal ResizeMessage_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class UserMessage;
struct UserMessageGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern UserMessageGlobalsTypeInternal UserMessage_globals_;
extern const ::google::protobuf::internal::ClassDataLite UserMessage_class_data_;
#else
extern const UserMessageGlobalsTypeInternal UserMessage_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace ClientBuffers
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace ClientBuffers {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ResizeMessage final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:ClientBuffers.ResizeMessage) */ {
  using Super_ = ::google::protobuf::MessageLite;

 public:
  inline ResizeMessage() : ResizeMessage(nullptr) {}
  ~ResizeMessage() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ResizeMessage* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ResizeMessage));
  }
#endif

  template <typename = void>
  explicit constexpr ResizeMessage(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ResizeMessage(const ResizeMessage& from) : ResizeMessage(nullptr, from) {}
  inline ResizeMessage(ResizeMessage&& from) noexcept : ResizeMessage(nullptr, ::std::move(from)) {}
  inline ResizeMessage& operator=(const ResizeMessage& from) {
    CopyFrom(from);
    return *this;
  }
  inline ResizeMessage& operator=(ResizeMessage&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const ResizeMessage& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ResizeMessage>(&ResizeMessage_globals_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(ResizeMessage& a, ResizeMessage& b) { a.Swap(&b); }
  inline void Swap(ResizeMessage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ResizeMessage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ResizeMessage* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<ResizeMessage>(arena);
  }
  void CopyFrom(const ResizeMessage& from);
  void MergeFrom(const ResizeMessage& from) { ResizeMessage::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ResizeMessage* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "ClientBuffers.ResizeMessage"; }

  explicit ResizeMessage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ResizeMessage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ResizeMessage& from);
  ResizeMessage(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ResizeMessage&& from) noexcept
      : ResizeMessage(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kWidthFieldNumber = 5,
    kHeightFieldNumber = 6,
  };
  // optional int32 width = 5;
  [[nodiscard]] bool has_width() const;
  void clear_width() ;
  [[nodiscard]] ::int32_t width() const;
  void set_width(::int32_t value);

  private:
  ::int32_t _internal_width() const;
  void _internal_set_width(::int32_t value);

  public:
  // optional int32 height = 6;
  [[nodiscard]] bool has_height() const;
  void clear_height() ;
  [[nodiscard]] ::int32_t height() const;
  void set_height(::int32_t value);

  private:
  ::int32_t _internal_height() const;
  void _internal_set_height(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:ClientBuffers.ResizeMessage)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ResizeMessage& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int32_t width_;
    ::int32_t height_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_userinput_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Keystroke final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:ClientBuffers.Keystroke) */ {
  using Super_ = ::google::protobuf::MessageLite;

 public:
  inline Keystroke() : Keystroke(nullptr) {}
  ~Keystroke() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Keystroke* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Keystroke));
  }
#endif

  template <typename = void>
  explicit constexpr Keystroke(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Keystroke(const Keystroke& from) : Keystroke(nullptr, from) {}
  inline Keystroke(Keystroke&& from) noexcept : Keystroke(nullptr, ::std::move(from)) {}
  inline Keystroke& operator=(const Keystroke& from) {
    CopyFrom(from);
    return *this;
  }
  inline Keystroke& operator=(Keystroke&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const Keystroke& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Keystroke>(&Keystroke_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(Keystroke& a, Keystroke& b) { a.Swap(&b); }
  inline void Swap(Keystroke* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Keystroke* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Keystroke* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Keystroke>(arena);
  }
  void CopyFrom(const Keystroke& from);
  void MergeFrom(const Keystroke& from) { Keystroke::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Keystroke* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "ClientBuffers.Keystroke"; }

  explicit Keystroke(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Keystroke(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Keystroke& from);
  Keystroke(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Keystroke&& from) noexcept
      : Keystroke(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kKeysFieldNumber = 4,
  };
  // optional bytes keys = 4;
  [[nodiscard]] bool has_keys() const;
  void clear_keys() ;
  [[nodiscard]] const ::std::string& keys() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_keys(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_keys();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_keys();
  void set_allocated_keys(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_keys() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_keys(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_keys();

  public:
  // @@protoc_insertion_point(class_scope:ClientBuffers.Keystroke)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Keystroke& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr keys_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_userinput_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Instruction final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:ClientBuffers.Instruction) */ {
  using Super_ = ::google::protobuf::MessageLite;

 public:
  inline Instruction() : Instruction(nullptr) {}
  ~Instruction() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Instruction* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Instruction));
  }
#endif

  template <typename = void>
  explicit constexpr Instruction(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Instruction(const Instruction& from) : Instruction(nullptr, from) {}
  inline Instruction(Instruction&& from) noexcept : Instruction(nullptr, ::std::move(from)) {}
  inline Instruction& operator=(const Instruction& from) {
    CopyFrom(from);
    return *this;
  }
  inline Instruction& operator=(Instruction&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const Instruction& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Instruction>(&Instruction_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(Instruction& a, Instruction& b) { a.Swap(&b); }
  inline void Swap(Instruction* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Instruction* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Instruction* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Instruction>(arena);
  }
  void CopyFrom(const Instruction& from);
  void MergeFrom(const Instruction& from) { Instruction::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Instruction* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "ClientBuffers.Instruction"; }

  explicit Instruction(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Instruction(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Instruction& from);
  Instruction(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Instruction&& from) noexcept
      : Instruction(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            typename = typename _proto_TypeTraits::Singular>
  PROTOBUF_FUTURE_ADD_NODISCARD inline bool HasExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id) const {
    return _impl_._extensions_.Has(id.number());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void ClearExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id) {
    _impl_._extensions_.ClearExtension(id.number());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            typename = typename _proto_TypeTraits::Repeated>
  PROTOBUF_FUTURE_ADD_NODISCARD inline int ExtensionSize(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id) const {
    return _impl_._extensions_.ExtensionSize(id.number());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            ::std::enable_if_t<!_proto_TypeTraits::kLifetimeBound, int> = 0>
  PROTOBUF_FUTURE_ADD_NODISCARD inline
      typename _proto_TypeTraits::Singular::ConstType
      GetExtension(const ::google::protobuf::internal::ExtensionIdentifier<
                   Instruction, _proto_TypeTraits, _field_type, _is_packed>& id)
          const {
    return _proto_TypeTraits::Get(id.number(), _impl_._extensions_, id.default_value());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            ::std::enable_if_t<_proto_TypeTraits::kLifetimeBound, int> = 0>
  PROTOBUF_FUTURE_ADD_NODISCARD inline
      typename _proto_TypeTraits::Singular::ConstType
      GetExtension(const ::google::protobuf::internal::ExtensionIdentifier<
                   Instruction, _proto_TypeTraits, _field_type, _is_packed>& id)
          const ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::Get(GetArena(), id.number(), _impl_._extensions_,
                                  id.default_value());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Singular::MutableType MutableExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id)
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::Mutable(GetArena(), id.number(), _field_type,
                                      &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void SetExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Singular::ConstType value) {
    _proto_TypeTraits::Set(GetArena(), id.number(), _field_type, value,
                           &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void SetAllocatedExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Singular::MutableType value) {
    _proto_TypeTraits::SetAllocated(GetArena(), id.number(), _field_type,
                                    value, &_impl_._extensions_);
  }
  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void UnsafeArenaSetAllocatedExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Singular::MutableType value) {
    _proto_TypeTraits::UnsafeArenaSetAllocated(
        GetArena(), id.number(), _field_type, value, &_impl_._extensions_);
  }
  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  [[nodiscard]] inline typename _proto_TypeTraits::Singular::MutableType
  ReleaseExtension(const ::google::protobuf::internal::ExtensionIdentifier<
                   Instruction, _proto_TypeTraits, _field_type, _is_packed>& id) {
    return _proto_TypeTraits::Release(GetArena(), id.number(), _field_type,
                                      &_impl_._extensions_);
  }
  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Singular::MutableType
  UnsafeArenaReleaseExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id) {
    return _proto_TypeTraits::UnsafeArenaRelease(
        GetArena(), id.number(), _field_type, &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            ::std::enable_if_t<!_proto_TypeTraits::kLifetimeBound, int> = 0>
  PROTOBUF_FUTURE_ADD_NODISCARD inline
      typename _proto_TypeTraits::Repeated::ConstType
      GetExtension(
          const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                           _field_type, _is_packed>& id,
          int index) const {
    return _proto_TypeTraits::Get(id.number(), _impl_._extensions_, index);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            ::std::enable_if_t<_proto_TypeTraits::kLifetimeBound, int> = 0>
  PROTOBUF_FUTURE_ADD_NODISCARD inline
      typename _proto_TypeTraits::Repeated::ConstType
      GetExtension(
          const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                           _field_type, _is_packed>& id,
          int index) const ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::Get(id.number(), _impl_._extensions_, index);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::MutableType MutableExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      int index) ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::Mutable(id.number(), index, &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void SetExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      int index, typename _proto_TypeTraits::Repeated::ConstType value) {
    _proto_TypeTraits::Set(id.number(), index, value, &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::MutableType AddExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id)
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    typename _proto_TypeTraits::Repeated::MutableType to_add =
        _proto_TypeTraits::Add(GetArena(), id.number(), _field_type,
                               &_impl_._extensions_);
    return to_add;
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void AddExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Repeated::ConstType value) {
    _proto_TypeTraits::Add(GetArena(), id.number(), _field_type, _is_packed,
                           value, &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  PROTOBUF_FUTURE_ADD_NODISCARD inline const typename _proto_TypeTraits::
      Repeated::RepeatedFieldType&
      GetRepeatedExtension(
          const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                           _field_type, _is_packed>& id)
          const ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::GetRepeated(id.number(), _impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::RepeatedFieldType* PROTOBUF_NONNULL
  MutableRepeatedExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Instruction, _proto_TypeTraits,
                                       _field_type, _is_packed>& id)
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::MutableRepeated(
        GetArena(), id.number(), _field_type, _is_packed, &_impl_._extensions_);
  }
  // @@protoc_insertion_point(class_scope:ClientBuffers.Instruction)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Instruction& from_msg);
    ::google::protobuf::internal::ExtensionSet _extensions_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_userinput_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED UserMessage final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:ClientBuffers.UserMessage) */ {
  using Super_ = ::google::protobuf::MessageLite;

 public:
  inline UserMessage() : UserMessage(nullptr) {}
  ~UserMessage() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(UserMessage* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(UserMessage));
  }
#endif

  template <typename = void>
  explicit constexpr UserMessage(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline UserMessage(const UserMessage& from) : UserMessage(nullptr, from) {}
  inline UserMessage(UserMessage&& from) noexcept : UserMessage(nullptr, ::std::move(from)) {}
  inline UserMessage& operator=(const UserMessage& from) {
    CopyFrom(from);
    return *this;
  }
  inline UserMessage& operator=(UserMessage&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const UserMessage& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<UserMessage>(&UserMessage_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(UserMessage& a, UserMessage& b) { a.Swap(&b); }
  inline void Swap(UserMessage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(UserMessage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] UserMessage* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<UserMessage>(arena);
  }
  void CopyFrom(const UserMessage& from);
  void MergeFrom(const UserMessage& from) { UserMessage::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(UserMessage* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "ClientBuffers.UserMessage"; }

  explicit UserMessage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  UserMessage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const UserMessage& from);
  UserMessage(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, UserMessage&& from) noexcept
      : UserMessage(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kInstructionFieldNumber = 1,
  };
  // repeated .ClientBuffers.Instruction instruction = 1;
  [[nodiscard]] int instruction_size() const;
  private:
  int _internal_instruction_size() const;

  public:
  void clear_instruction() ;
  [[nodiscard]] const ::ClientBuffers::Instruction& instruction(int index) const;
  [[nodiscard]] ::ClientBuffers::Instruction* PROTOBUF_NONNULL mutable_instruction(int index);
  ::ClientBuffers::Instruction* PROTOBUF_NONNULL add_instruction();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::ClientBuffers::Instruction>&
  instruction() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::ClientBuffers::Instruction>* PROTOBUF_NONNULL
  mutable_instruction();

  private:
  const ::google::protobuf::RepeatedPtrField<::ClientBuffers::Instruction>& _internal_instruction() const;
  ::google::protobuf::RepeatedPtrField<::ClientBuffers::Instruction>* PROTOBUF_NONNULL _internal_mutable_instruction();

  public:
  // @@protoc_insertion_point(class_scope:ClientBuffers.UserMessage)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const UserMessage& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::ClientBuffers::Instruction > instruction_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_userinput_2eproto;
};

// ===================================================================



inline constexpr int kKeystrokeFieldNumber = 2;
extern ::google::protobuf::internal::ExtensionIdentifier<
    ::ClientBuffers::Instruction, ::google::protobuf::internal::MessageTypeTraits< ::ClientBuffers::Keystroke >, 11, false>(keystroke);
inline constexpr int kResizeFieldNumber = 3;
extern ::google::protobuf::internal::ExtensionIdentifier<
    ::ClientBuffers::Instruction, ::google::protobuf::internal::MessageTypeTraits< ::ClientBuffers::ResizeMessage >, 11, false>(resize);

// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// UserMessage

// repeated .ClientBuffers.Instruction instruction = 1;
inline int UserMessage::_internal_instruction_size() const {
  return _internal_instruction().size();
}
inline int UserMessage::instruction_size() const {
  return _internal_instruction_size();
}
inline void UserMessage::clear_instruction() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.instruction_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::ClientBuffers::Instruction& UserMessage::instruction(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:ClientBuffers.UserMessage.instruction)
  return _internal_instruction().Get(index);
}
inline ::ClientBuffers::Instruction* PROTOBUF_NONNULL UserMessage::mutable_instruction(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:ClientBuffers.UserMessage.instruction)
  return _internal_mutable_instruction()->Mutable(index);
}
inline ::ClientBuffers::Instruction* PROTOBUF_NONNULL UserMessage::add_instruction()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::ClientBuffers::Instruction* _add =
      _internal_mutable_instruction()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:ClientBuffers.UserMessage.instruction)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::ClientBuffers::Instruction>& UserMessage::instruction() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:ClientBuffers.UserMessage.instruction)
  return _internal_instruction();
}
inline ::google::protobuf::RepeatedPtrField<::ClientBuffers::Instruction>* PROTOBUF_NONNULL
UserMessage::mutable_instruction() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:ClientBuffers.UserMessage.instruction)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_instruction();
}
inline const ::google::protobuf::RepeatedPtrField<::ClientBuffers::Instruction>&
UserMessage::_internal_instruction() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.instruction_;
}
inline ::google::protobuf::RepeatedPtrField<::ClientBuffers::Instruction>* PROTOBUF_NONNULL
UserMessage::_internal_mutable_instruction() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.instruction_;
}

// -------------------------------------------------------------------

// Instruction

// -------------------------------------------------------------------

// Keystroke

// optional bytes keys = 4;
inline bool Keystroke::has_keys() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void Keystroke::clear_keys() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.keys_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Keystroke::keys() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:ClientBuffers.Keystroke.keys)
  return _internal_keys();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Keystroke::set_keys(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.keys_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:ClientBuffers.Keystroke.keys)
}
inline ::std::string* PROTOBUF_NONNULL Keystroke::mutable_keys()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_keys();
  // @@protoc_insertion_point(field_mutable:ClientBuffers.Keystroke.keys)
  return _s;
}
inline const ::std::string& Keystroke::_internal_keys() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.keys_.Get();
}
inline void Keystroke::_internal_set_keys(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.keys_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Keystroke::_internal_mutable_keys() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.keys_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Keystroke::release_keys() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:ClientBuffers.Keystroke.keys)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.keys_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.keys_.Set("", GetArena());
  }
  return released;
}
inline void Keystroke::set_allocated_keys(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.keys_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.keys_.IsDefault()) {
    _impl_.keys_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:ClientBuffers.Keystroke.keys)
}

// -------------------------------------------------------------------

// ResizeMessage

// optional int32 width = 5;
inline bool ResizeMessage::has_width() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void ResizeMessage::clear_width() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.width_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::int32_t ResizeMessage::width() const {
  // @@protoc_insertion_point(field_get:ClientBuffers.ResizeMessage.width)
  return _internal_width();
}
inline void ResizeMessage::set_width(::int32_t value) {
  _internal_set_width(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:ClientBuffers.ResizeMessage.width)
}
inline ::int32_t ResizeMessage::_internal_width() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.width_;
}
inline void ResizeMessage::_internal_set_width(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.width_ = value;
}

// optional int32 height = 6;
inline bool ResizeMessage::has_height() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void ResizeMessage::clear_height() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.height_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int32_t ResizeMessage::height() const {
  // @@protoc_insertion_point(field_get:ClientBuffers.ResizeMessage.height)
  return _internal_height();
}
inline void ResizeMessage::set_height(::int32_t value) {
  _internal_set_height(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:ClientBuffers.ResizeMessage.height)
}
inline ::int32_t ResizeMessage::_internal_height() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.height_;
}
inline void ResizeMessage::_internal_set_height(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.height_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace ClientBuffers


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // userinput_2eproto_2epb_2eh
